# Domoticz WiZ connected Plugin
#
# Author: Syds Post sydspost@gmail.com
# Author: Steel Rat 
#
"""
<plugin key="wiz" name="WiZ connected" author="Steel Rat" version="1.0.1" wikilink="" externallink="https://www.wizconnected.com/">
    <description>
        <h2>WiZ connected Plugin</h2><br/>
        Fork from  Syds Post sydspost@gmail.com
        This plugin is meant to control WiZ connected devices. WiZ connected devices may come with different brands as Philips, TAO and WiZ connected etc.
        <h3>Features</h3>
        <ul style="list-style-type:square">
            <li>Auto-detection of devices on network</li>
            <li>On/Off control, state and available status display</li>
            <li>Dimmer/Warm-cold setting for Lights</li>
<!--            <li>Scene activation support</li> -->
        </ul>
        <h3>Devices</h3>
        <ul style="list-style-type:square">
            <li>All devices that have on/off state should be supported</li>
        </ul>
        <h3>Configuration</h3>
        Devices can be renamed in Domoticz or you can rename them in the App and remove them from Domoticz so they are detected with a new name or layout.
        Be careful when setting the subnet, setting class A or class B networks can lead to performance issues because the plugin uses ARP ping
        to detect the WiZ connected devices. Hostname Prefix is a comma seperated list of the first 4 characters of the hostname of the WiZ connected devices per brand.
    </description>
    <params>
        <param field="Mode1" label="Hostname Prefix" width="200px" required="true" default="wiz_"/>
        <param field="Mode2" label="Subnet" width="200px" required="true" default="192.168.2.0/24"/>
        <param field="Mode3" label="Language" width="200px">
            <options>
                <option label="English" value="EN" default="true" />
                <option label="Polski" value="PL" />
            </options>
        </param>
        <param field="Mode6" label="Debug" width="150px">
            <options>
                <option label="None" value="0"  default="true" />
                <option label="Python Only" value="2"/>
                <option label="Basic Debugging" value="62"/>
                <option label="Basic+Messages" value="126"/>
                <option label="Connections Only" value="16"/>
                <option label="Connections+Python" value="18"/>
                <option label="Connections+Queue" value="144"/>
                <option label="All" value="-1"/>
            </options>
        </param>
    </params>
</plugin>
"""
import Domoticz
import threading
import socket
import html
import sys
import time
import math
import json
import re
from scapy.all import srp,Ether,ARP,conf

class BasePlugin:
    startup = True;
    devs = {}
    last_update = 0

    def __init__(self):
        return

    def onStart(self):
        Domoticz.Log("WiZ connected plugin started")
        if Parameters["Mode6"] != "0":
            Domoticz.Debugging(int(Parameters["Mode6"]))
            DumpConfigToLog()

        # Find devices that already exist, create those that don't
        conf.verb=0
        ans,unans=srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=Parameters["Mode2"]), timeout=2)

        hostNames = []

        for snd,rcv in ans:
            ipAddress=rcv.sprintf(r"%ARP.psrc%")
            try:
                hostName = socket.gethostbyaddr(ipAddress)
            except:
                hostName = "Onbekend"

            if hostName[0][0:3] in Parameters["Mode1"]:
               hostNames.append(hostName[0])
               Domoticz.Log("HostName:" + hostName[0] )
               unitnumber = getUnit(hostName[0])
               if unitnumber == 0:
                  CreateDomoticzDevice(hostName[0], len(Devices)+1, GetLampType(hostName[0]), "")

                  CreateDomoticzDevice(hostName[0], len(Devices)+1, "Generic1", DecodeDeviceType(Devices[len(Devices)].Type, Devices[len(Devices)].SubType, Devices[len(Devices)].SwitchType ))
                  CreateDomoticzDevice(hostName[0], len(Devices)+1, "Generic2", "")
                  Domoticz.Log("HostName:" + hostName[0]+" Devices:" + str(len(Devices)) )
               else:
                  Domoticz.Log("HostName:" + hostName[0]+" Devices:" + str(len(Devices)) )
                  _unitnumber = (3 * (int((Devices[unitnumber].Unit - 1)/3))) + 1
                  Domoticz.Log("HostName:" + hostName[0]+" unit:" + str( _unitnumber) )
                  try:
                      a = Devices[_unitnumber].DeviceID
                  except:
                      CreateDomoticzDevice(hostName[0], _unitnumber, GetLampType(hostName[0]), "")
                  try:
                      a = Devices[_unitnumber+1].DeviceID
                  except:
                      CreateDomoticzDevice(hostName[0], _unitnumber+1, "Generic1", DecodeDeviceType(Devices[_unitnumber].Type,Devices[_unitnumber].SubType, Devices[_unitnumber].SwitchType))
                  try:
                      a = Devices[_unitnumber+2].DeviceID
                  except:
                      CreateDomoticzDevice(hostName[0], _unitnumber+2, "Generic2", "")

        # Create/Start update thread
        self.updateThread = threading.Thread(name="WiZUpdateThread", target=BasePlugin.handleThread, args=(self,))
        self.updateThread.start()

    def onStop(self):
        Domoticz.Debug("onStop called")
        while (threading.active_count() > 1):
            time.sleep(1.0)

    def onConnect(self, Connection, Status, Description):
        Domoticz.Debug("onConnect called")

    def onMessage(self, Connection, Data):
        Domoticz.Debug("onMessage called")

    def onCommand(self, Unit, Command, Level, Color):
        Domoticz.Debug("onCommand called for Unit " + str(Unit) + ": Parameter '" + str(Command) + "', Level: " + str(Level))

        # Find the device for the Domoticz Unit provided
        dev = None
        try:
            dev = Devices[Unit]
        except Exception as e:
            Domoticz.Debug("Device has no ID " + str(Unit) + " " + str(e))

        # If we didn't find it, leave (probably disconnected at this time)
        if dev == None:
            Domoticz.Error('Command for DeviceID='+Devices[Unit].DeviceID+' but device is not available.')
            return

        # if not dev.available():
        #     Domoticz.Error('Command for DeviceID='+Devices[Unit].DeviceID+' but device is offline.')
        #     return

        Domoticz.Log('Sending command for DeviceID='+Devices[Unit].DeviceID+ ' ' +Command)

        # Control device and update status in Domoticz
        if Command == 'On':
            UpdateDevice(Unit, 1, 'On', not SetLampCommand(Devices[Unit].DeviceID, b'{"method":"setPilot","params":{"src":"udp","state":true}}'))
        elif Command == 'Off':
            UpdateDevice(Unit, 0, 'Off', not SetLampCommand(Devices[Unit].DeviceID, b'{"method":"setPilot","params":{"src":"udp","state":false}}'))
        elif Command == 'Set Color':
            # Convert RGB to Cold- and White level
            rgb = json.loads(Color)
            mode = rgb.get("m")
            cw = rgb.get("cw")
            ww = rgb.get("ww")
            r = rgb.get("r")
            g = rgb.get("g")
            b = rgb.get("b")

            if mode == 2:
                Domoticz.Log('Color mode 2 : '+Devices[Unit].DeviceID+ ' ' +Command)
                Devices[Unit].Update(nValue=1, sValue=str(Level), TimedOut= not SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"c":' + str(cw) + ',"w":' + str(ww) + ',"dimming":' + str(Level) + '}}', 'utf-8')), Color=Color)
                
            if mode == 3:
                Domoticz.Log('Color mode 3 : '+Devices[Unit].DeviceID+ ' ' +Command)
                Devices[Unit].Update(nValue=1, sValue=str(Level), TimedOut= not SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"r":' + str(r) + ',"g":' + str(g)+ ',"b":' + str(b) + ',"dimming":' + str(Level) +'}}', 'utf-8')), Color=Color)
        elif Command == 'Set Level':
            CurentUnit = int((Unit-1) / 3)
            FirstUnit = (3*CurentUnit)+1
            SecondUnit =(3*CurentUnit)+2
            ThirdUnit = (3*CurentUnit)+3
            deviceType = DecodeDeviceType(Devices[FirstUnit].Type,Devices[FirstUnit].SubType,Devices[FirstUnit].SwitchType)
            # Set new level
            if (Unit==FirstUnit):
                Domoticz.Log('Sending command for DeviceID='+Devices[Unit].DeviceID+ ' ' +Command)
                UpdateDevice(Unit, 1 if Devices[Unit].Type == 241 else 2, str(Level), not SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"dimming":' + str(Level) + '}}', 'utf-8')))

                Devices[SecondUnit].Update(nValue = 1, sValue = "10")
                Devices[ThirdUnit].Update(nValue = 0,sValue = Devices[ThirdUnit].sValue)
            scena = 0
            if (Unit==SecondUnit):
                #if (Level == 0): mJSON1 = b'{"method":"setPilot","params":{"src":"udp","state":false}}'

                Devices[Unit].Update(nValue = Devices[Unit].nValue, sValue = str(Level))
                if (DecodeHaveSpeed(deviceType, str(Level))):
                    Devices[ThirdUnit].Update(nValue = 1, sValue = Devices[ThirdUnit].sValue)
                else:
                    Devices[ThirdUnit].Update(nValue = 0,sValue = Devices[ThirdUnit].sValue)

            if (Unit == ThirdUnit):
                Devices[Unit].Update(nValue = Devices[Unit].nValue, sValue = str(Level))

            if (Devices[ThirdUnit].nValue == 1 and CurentUnit != FirstUnit):
                if (Devices[ThirdUnit].sValue == "10"): speed = 10
                if (Devices[ThirdUnit].sValue == "20"): speed = 50
                if (Devices[ThirdUnit].sValue == "30"): speed = 100
                if (Devices[ThirdUnit].sValue == "40"): speed = 150
                if (Devices[ThirdUnit].sValue == "50"): speed = 200
                scena = ModSceneID(deviceType, Devices[SecondUnit].sValue, True)
                SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":false,"sceneId":' +str(scena) + ',"speed":'+str(speed)+'}}', 'utf-8'))
                time.sleep(1)
                SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"sceneId":' +str(scena) + ',"speed":'+str(speed)+'}}', 'utf-8'))

            elif (Devices[ThirdUnit].nValue == 0 and CurentUnit != FirstUnit):
                scena = ModSceneID(deviceType, Devices[SecondUnit].sValue, True)
                SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"sceneId":'+str(scena)+'}}', 'utf-8'))

        elif Command == 'Set Night':
            # Set new level
            # Update status of Domoticz device
            Devices[Unit].Update(nValue=1, sValue=str(Level), TimedOut=not SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"sceneId":14}}', 'utf-8')), Color=Color)
        elif Command == 'Set Full':
            # Set new level
            # Update status of Domoticz device
            Devices[Unit].Update(nValue=1, sValue=str(Level), TimedOut=not SetLampCommand(Devices[Unit].DeviceID, bytes('{"method":"setPilot","params":{"src":"udp","state":true,"sceneId":12}}', 'utf-8')), Color=Color)
        # Set last update
        self.last_update = time.time()

    def onNotification(self, Name, Subject, Text, Status, Priority, Sound, ImageFile):
        Domoticz.Debug("Notification: " + Name + "," + Subject + "," + Text + "," + Status + "," + str(Priority) + "," + Sound + "," + ImageFile)

    def onDisconnect(self, Connection):
        Domoticz.Debug("onDisconnect called")

    def onHeartbeat(self):
        Domoticz.Debug("onHeartbeat called time="+str(time.time()))
        # If it hasn't been at least 1 minute (corrected for ~2s runtime) since last update, skip it
        if time.time() - self.last_update < 30:
            return

        # Create/Start update thread
        self.updateThread = threading.Thread(name="WiZUpdateThread", target=BasePlugin.handleThread, args=(self,))
        self.updateThread.start()

    # Separate thread looping ever 10 seconds searching for new WiZ connected devices on network and updating their status
    def handleThread(self):
        try:

            Domoticz.Debug("in handlethread")
            # Initialize/Update WiZ devices

            # Set last update
            self.last_update = time.time()

            # Update devices
            conf.verb=0
            ans,unans=srp(Ether(dst="ff:ff:ff:ff:ff:ff")/ARP(pdst=Parameters["Mode2"]), timeout=2)

            hostNames = []
            temperatureLamp = [0,0,1]

            for snd,rcv in ans:
                ipAddress=rcv.sprintf(r"%ARP.psrc%")
                try:
                    hostName = socket.gethostbyaddr(ipAddress)
                except:
                    hostName = "Onbekend"

                if hostName[0][0:3] in Parameters["Mode1"]:
                    hostNames.append(hostName[0])
                    Domoticz.Log("HostName:" + hostName[0] )
                    unitnumber = getUnit(hostName[0])

                    if unitnumber == 0:
                       CreateDomoticzDevice(hostName[0], len(Devices)+1, GetLampType(hostName[0]), "")
                       CreateDomoticzDevice(hostName[0], len(Devices)+1, "Generic1", DecodeDeviceType(Devices[len(Devices)].Type,Devices[len(Devices)].SubType, Devices[len(Devices)].SwitchType))
                       CreateDomoticzDevice(hostName[0], len(Devices)+1, "Generic2", "")
                       Domoticz.Log("HostName:" + hostName[0]+" Devices:" + str(len(Devices)) )
                    else:
                       Domoticz.Log("HostName:" + hostName[0]+" Devices:" + str(len(Devices)) )
                       _unitnumber = (3 * (int((Devices[unitnumber].Unit - 1)/3))) + 1

                       Domoticz.Log("HostName:" + hostName[0]+" unit:" + str( _unitnumber) )
                       try:
                           a = Devices[_unitnumber].DeviceID
                       except:
                           CreateDomoticzDevice(hostName[0], _unitnumber, GetLampType(hostName[0]), "")
                       try:
                           a = Devices[_unitnumber+1].DeviceID
                       except:
                           CreateDomoticzDevice(hostName[0], _unitnumber+1, "Generic1", DecodeDeviceType(Devices[_unitnumber].Type,Devices[_unitnumber].SubType, Devices[_unitnumber].SwitchType))
                       try:
                           a = Devices[_unitnumber+2].DeviceID
                       except:
                           CreateDomoticzDevice(hostName[0], _unitnumber+2, "Generic2", "")

            for Device in Devices:
                CurentDevice = int((Device - 1) / 3)
                FirstDevice = (3*CurentDevice)+1
                if (Device != FirstDevice): continue
                SecondDevice =(3*CurentDevice)+2
                ThirdDevice = (3*CurentDevice)+3
                received = GetLampStatus(Devices[FirstDevice].DeviceID)
                deviceType = DecodeDeviceType(Devices[FirstDevice].Type, Devices[FirstDevice].SubType, Devices[FirstDevice].SwitchType)

                if (received != ""):
                    temperatureLamp = SetLampTemp(GetLampType(Devices[FirstDevice].DeviceID))
                    Domoticz.Log("Domoticz color -11 =" + str(Devices[FirstDevice].Color))
                    wizstate=received["result"]["state"]

                    if wizstate:
                       wizstate = 1
                    else:
                       wizstate = 0
                    
                    try:
                      wizlevel=str(received["result"]["dimming"])
                    except:
                      wizlevel="10"

                    sceneID = ModSceneID(deviceType, received["result"]["sceneId"], False)
                    if (wizstate == 0): sceneID=0
                    Domoticz.Log("Domoticz color -10 =" + str(Devices[FirstDevice].Color))
                    try:
                        if (received["result"]["speed"]  <=24): speedlevel = 10
                        if (received["result"]["speed"]  >27 and received["result"]["speed"]  <=72): speedlevel = 20
                        if (received["result"]["speed"]  >72 and received["result"]["speed"]  <=120): speedlevel = 30
                        if (received["result"]["speed"]  >120 and received["result"]["speed"]  <=168): speedlevel = 40
                        if (received["result"]["speed"]  >168): speedlevel = 50
                        Devices[ThirdDevice].Update(nValue=1, sValue=str(speedlevel), TimedOut = False)
                    except:
                        speedlevel = 30
                        Devices[ThirdDevice].Update(nValue=0, sValue=str(speedlevel), TimedOut = False)

                    if  (DecodeHaveSpeed(deviceType, Devices[SecondDevice].sValue) == False or wizstate == 0):
                        speedlevel = 30
                        Devices[ThirdDevice].Update(nValue=0, sValue=str(speedlevel), TimedOut = False)

                    if (sceneID == 10):
                        Domoticz.Log("SceneID =" + str(sceneID))
                        Domoticz.Log("Domoticz first device =" + str(FirstDevice))
                        Domoticz.Log("Domoticz color 0 =" + str(Devices[FirstDevice].Color))
                        Color1 =' {"b":0,"cw":0,"g":0,"m":2,"r":0,"t":0,"ww":0}'
                        Domoticz.Log("Domoticz color 1 =" + str(Devices[FirstDevice].Color))
                        c=json.loads(Color1)
                        try:
                            wizcw=received["result"]["c"]
                            wizww=received["result"]["w"]
                            wizr=received["result"]["r"]
                            wizg=received["result"]["g"]
                            wizb=received["result"]["b"]

                            c=json.loads(Color1)
                            if wizr == 0 and wizg == 0 and wizb == 0:
                               c['m'] = 2
                               c['cw'] = wizcw
                               c['ww'] = wizww
                            else:
                               c['m'] = 3
                               c["r"] = wizr
                               c["g"] = wizg
                               c["b"] = wizb
                        except:
                            c=json.loads(Color1)
                            c['m'] = 2
                            wiztemp=received["result"]["temp"]
                            c["t"] = 255 - round((wiztemp - temperatureLamp[0]) / temperatureLamp[2])
                        wizcolor=json.dumps(c)
                    else:
                       wizcolor=""

                # Update status of Domoticz device
                    Domoticz.Log("Wizcolor temp =" + str(wizcolor))
                    Devices[FirstDevice].Update(nValue=wizstate, sValue=wizlevel, TimedOut=False, Color=str(wizcolor))
                    Devices[SecondDevice].Update(nValue=wizstate, sValue=str(sceneID), TimedOut=False)
                    #Devices[ThirdDevice].Update(nValue=Devices[ThirdDevice].nValue, sValue=Devices[ThirdDevice].sValue, TimdeOut=False)

                else:
                    a = 1
                    Devices[FirstDevice].Update(nValue=0, sValue="0", TimedOut=True, Color=Devices[FirstDevice].Color)
                    Devices[SecondDevice].Update(nValue=0, sValue="0", TimedOut=True)
                    Devices[ThirdDevice].Update(nValue=0, sValue="30", TimedOut=True)


        except Exception as err:
            Domoticz.Error("handleThread1: "+str(err)+' line '+format(sys.exc_info()[-1].tb_lineno))


global _plugin
_plugin = BasePlugin()

def onStart():
    global _plugin
    _plugin.onStart()

def onStop():
    global _plugin
    _plugin.onStop()

def onConnect(Connection, Status, Description):
    global _plugin
    _plugin.onConnect(Connection, Status, Description)

def onMessage(Connection, Data):
    global _plugin
    _plugin.onMessage(Connection, Data)

def onCommand(Unit, Command, Level, Color):
    global _plugin
    _plugin.onCommand(Unit, Command, Level, Color)

def onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile):
    global _plugin
    _plugin.onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile)

def onDisconnect(Connection):
    global _plugin
    _plugin.onDisconnect(Connection)

def onHeartbeat():
    global _plugin
    _plugin.onHeartbeat()

    # Generic helper functions
def DumpConfigToLog():
    for x in Parameters:
        if Parameters[x] != "":
            Domoticz.Debug( "'" + x + "':'" + str(Parameters[x]) + "'")
    Domoticz.Debug("Device count: " + str(len(Devices)))
    for x in Devices:
        Domoticz.Debug("Device:           " + str(x) + " - " + str(Devices[x]))
        Domoticz.Debug("Device ID:       '" + str(Devices[x].ID) + "'")
        Domoticz.Debug("Device Name:     '" + Devices[x].Name + "'")
        Domoticz.Debug("Device nValue:    " + str(Devices[x].nValue))
        Domoticz.Debug("Device sValue:   '" + Devices[x].sValue + "'")
        Domoticz.Debug("Device LastLevel: " + str(Devices[x].LastLevel))
    return

# Loop thru domoticz devices and see if there's a device with matching DeviceID, if so, return unit number, otherwise return zero
def getUnit(devid):
    unit = 0
    for x in Devices:
        if Devices[x].DeviceID == devid:
            unit = x
            break
    return unit

# Find the smallest unit number available to add a device in domoticz
def nextUnit():
    unit = 1
    while unit in Devices and unit < 255:
        unit = unit + 1
    return unit

def UpdateDevice(Unit, nValue, sValue, TimedOut):
    # Make sure that the Domoticz device still exists (they can be deleted) before updating it
    if (Unit in Devices):
        #if (Devices[Unit].nValue != nValue) or (Devices[Unit].sValue != sValue) or (Devices[Unit].TimedOut != TimedOut):
        Devices[Unit].Update(nValue=nValue, sValue=str(sValue), TimedOut=TimedOut)
        Domoticz.Log("Update "+str(nValue)+":'"+str(sValue)+"' ("+Devices[Unit].Name+") TimedOut="+str(TimedOut))
    return

def SetLampTemp(_device):
    #6500-2200=4300  4300/255=16.86
    if (_device == "ESP24_SHRGBC_01"):return [2200,6500,16.86] 
    if (_device == "ESP01_SHRGB_03"):return [2200,6500,16.86] 
    if (_device ==  "ESP03_SHTW1C_01"):return [2700,6500,14.9]
    return [2200,2200,1]

def GetSelectorNames(_lang, _seltype):
    if (_lang == "EN"):
        if (_seltype == "RGBWW"): return "off|Unknown|White heat|Daylight|Cold white|Night lamp|Home Comfort|True colors|Meditation|Greenhouse|Focus|Film|Light of candles|Fireplace|Autumn|Klub|Sunset|Romantic|Party|Pastel colors|Spring|Summer|Forest|Jungle|Mojito|Ocean|Immersion|Pulsating|Gold white|Steampunk|Refuge|Before bedtime|Sylwester|Hallowen"
        if (_seltype == "WW"):return "off|Unknown|White heat|Daylight|Cold white|Night lamp|Home Comfort|Meditation|Focus|Film|Refuge|Before bedtime|Light of candles|Pulsating|Gold white|Steampunk"
        if (_seltype == "W"):return "off|Unknown|White|Night lamp|Refuge|Before bedtime|Light of candles|Pulsating|Steampunk"
    if (_lang == "PL"):
        if (_seltype == "RGBWW"): return "off|Nieznana|Ciepła biel|Światło dzienne|Zimna biel|Lampka nocna|Komfort domowy|Prawdziwe kolory|Medytacja|Szklarnia|Skupienie|Film|Światło świec|Kominek|Jesień|Klub|Zachód słońca|Romantyka|Impreza|Kolory pastelowe|Wiosna|Lato|Las|Dżungla|Mojito|Ocean|Zanurzenie|Pulsujący|Złociście-biały|Steampunk|Pobudka|Przed snem|Sylwester|Hallowen"
        if (_seltype == "WW"):return "off|Nieznana|Ciepła biel|Światło dzienne|Zimna biel|Lampka nocna|Komfort domowy|Medytacja|Skupienie|Film|Pobudka|Przed snem|Światło świec|Pulsujący|Złociście-biały|Steampunk"
        if (_seltype == "W"):return "off|Nieznana|Biały|Lampka nocna|Pobudka|Przed snem|Światło świec|Pulsujący|Steampunk"
    return ""

def CreateDomoticzDevice(_hostName, _device , _devType, _subtype):
        if (_devType == "ESP24_SHRGBC_01"):
             Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=241, Subtype=4, Switchtype=7, Image=0).Create()
        if (_devType == "ESP01_SHRGB_03"):
             Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=241, Subtype=4, Switchtype=7, Image=0).Create()
        if (_devType == "ESP03_SHDW1_01"):
             Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=244, Subtype=73, Switchtype=7, Image=0).Create()
        if (_devType == "ESP03_SHTW1C_01"):
             Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=241, Subtype=8, Switchtype=7, Image=0).Create()
        if (_devType == "Generic1"):
            if (_subtype == "RGBWW"):
                SelectorNames=GetSelectorNames(Parameters["Mode3"], _subtype)
                Domoticz.Log("Parametr mode3 : " + Parameters["Mode3"])
                SelectorOptions="|||||||||||||||||||||||||||||||||"
                Options={"LevelNames":SelectorNames,
                         "LevelActions":SelectorOptions,
                         "LevelOffHidden":"false",
                         "SelectorStyle":"1"}
            if (_subtype == "WW"):
                SelectorNames=GetSelectorNames(Parameters["Mode3"], _subtype)
                SelectorOptions="|||||||||||||||"
                Options={"LevelNames":SelectorNames,
                         "LevelActions":SelectorOptions,
                         "LevelOffHidden":"false",
                         "SelectorStyle":"1"}
            if (_subtype == "W"):
                SelectorNames=GetSelectorNames(Parameters["Mode3"], _subtype)
                SelectorOptions="||||||||"
                Options={"LevelNames":SelectorNames,
                         "LevelActions":SelectorOptions,
                          "LevelOffHidden":"false",
                          "SelectorStyle":"1"}
            Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=244, Subtype=73, Switchtype=18, Image=0, Options=Options).Create()
        if (_devType == "Generic2"):
            SelectorNames="off|min|25%|50%|75%|max"
            SelectorOptions="|||||"
            Options={"LevelNames":SelectorNames,
                     "LevelActions":SelectorOptions,
                     "LevelOffHidden":"true",
                     "SelectorStyle":"0"}
            Domoticz.Device(Name=_hostName, DeviceID=_hostName,  Unit=_device, Type=244, Subtype=73, Switchtype=18, Image=0, Options=Options).Create()
        return

def GetLampType(_hostID):
    host = _hostID
    port = 38899
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(2)
    _mJSON = b'{"method":"getDevInfo"}'
    _wiztype = ""
    try:
        sock.sendto(_mJSON, (host, port))
        received = sock.recv(1024).decode('utf-8')
        received=json.loads(received)
        _wiztype=received["result"]["moduleName"]
    except:
        return _wiztype
    finally:
        sock.close()
    return _wiztype


def SetLampCommand(_hostID, _mJSON):
    host = str(_hostID)
    port = 38899
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(2)
    _received = False
    try:
        sock.sendto(_mJSON, (host, port))
        _received = sock.recv(1024).decode('utf-8')
        _received=str(_received).split(",")[2].split(":")[2]
        _received=_received[0:len(_received)-3].capitalize()
    except:
         _received = False
    finally:
        sock.close()

    return _received

def GetLampStatus(_hostID):
    host = str(_hostID)
    port = 38899
    _received = ""
    _mJSON = b'{"method":"getPilot"}'
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.settimeout(2)

    try:
        sock.sendto(_mJSON, (host, port))
        _received = sock.recv(1024).decode('utf-8')
        _received=json.loads(_received)
    except: 
        _received = ""
    finally:
         sock.close()

    return _received

def DecodeDeviceType(_type, _subtype, _switchtype):
    if (_type == 241 and _subtype == 4 and _switchtype == 7): return "RGBWW"
    if (_type == 244 and _subtype == 73 and _switchtype == 7): return "W"
    if (_type == 241 and _subtype == 8 and _switchtype == 7): return "WW"
    return ""

def DecodeHaveSpeed(_type, _level):
    if _level == "": _level = 0
    if (_type == "RGBWW"):
        _levels = [0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,1,1]
    #if (_type == "W"):
    #    if (_level == 70 or _level == 80): return True
    if (_type == "WW"):
        _levels = [0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1]
    if (_levels[int(int(_level)/10)] == 1):
        return True
    else:
        return False

def ModSceneID(_type, _level, _set):
    if (_level == ""): _level = 0
    if (_type == "RGBWW"):
        _levels = [-1,0,11,12,13,14,6,17,16,19,15,18,29,5,22,26,3,2,4,8,20,21,7,24,25,1,23,31,30,32,9,10,27,28]
    if (_type == "WW"):
        _levels = [-1,0,11,12,13,14,6,16,15,17,9,10,29,31,30,32]
    if (_type == "W"):
        _levels = [-1,0,12,14,9,10,29,31,32]
    if (_set == True):
        return _levels[int(int(_level)/10)]
    else:
        return _levels.index(int(_level))*10